﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Drawing;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Media.Imaging;


//This code was written by a person called Chris van Beek. I came across his code when I was reading his blogg post about him
//creating an application where he gets a livefeed from a similar camera that I was using. The address to his webpage
//is:https://blogs.infosupport.com/writing-an-ip-camera-viewer-in-c-5-0/ where he does a good job explaining his work. 
//You can also download his original sorce code from here.
namespace cCam
{
    class IpCamController
    {
        private string _url;
        private string _videoRelative = "/videostream.cgi?resolution=32&rate=0";
        private HttpClient _client;
        private AutomaticMultiPartReader _reader;
        private BitmapImage _currentFrame;
        public BitmapImage CurrentFrame
        {
            get { return _currentFrame; }
        }
        private frmLogForm myLog;

        

        public IpCamController(string url, string username, string password)
        {
            _url = url;
            WebRequestHandler handler = new WebRequestHandler();
            handler.Credentials = new NetworkCredential(username, password);
            _client = new HttpClient(handler);
            _client.BaseAddress = new Uri(_url);
            _client.Timeout = TimeSpan.FromMilliseconds(-1);
        }

        private void _reader_PartReady(object sender, PartReadyEventArgs e)
        {
            //let's get this events back on the UI thread
            Stream frameStream = new MemoryStream(e.Part);
            App.Current.Dispatcher.Invoke(new Action(() =>
            {
                _currentFrame = new BitmapImage();
                _currentFrame.BeginInit();
                _currentFrame.StreamSource = frameStream;
                _currentFrame.EndInit();
                OnImageReady();
            }));
        }

        public async void StartProcessing()
        {
            HttpResponseMessage resultMessage = await _client.GetAsync(_videoRelative, HttpCompletionOption.ResponseHeadersRead);
            //because of the configure await the rest of this method happens on a background thread.
            resultMessage.EnsureSuccessStatusCode();
            // check the response type
            if (!resultMessage.Content.Headers.ContentType.MediaType.Contains("multipart"))
            {
                throw new ArgumentException("The camera did not return a mjpeg stream");
            }
            else
            {
                _reader = new AutomaticMultiPartReader(new MultiPartStream(await resultMessage.Content.ReadAsStreamAsync()));
                _reader.PartReady += _reader_PartReady;
                _reader.StartProcessing();
            }
        }

        public void StopProcessing()
        {
            _reader.StopProcessing();
        }

        protected void OnImageReady()
        {
            if (ImageReady != null)
            {
                ImageReady(this, new ImageReadyEventArsgs() { Image = _currentFrame/*save*/ });
            }
        }

        public event EventHandler<ImageReadyEventArsgs> ImageReady;

    }
}
