﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows;
using System.Windows.Threading;
using System.Windows.Forms;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

//Purpos: Give the user the option to watch the live feed from the camera, start and stop motion detection
//and take a snapshot. 
namespace cCam
{
    
    public partial class LivePage : Window
    {
        BitmapImage SnapShot;
        BitmapImage Base_Pic;
        BitmapImage Current_Motion;
        Bitmap Base_Try;
        Bitmap Current_Try;
        private IpCamController _controller;
        private int Timer = 0;
        private int Timer_color = 0;
        private DispatcherTimer snap_color;
        private DispatcherTimer New_Base_Pic_Timer;
        private DispatcherTimer Comp_Images;
        private int close_motion = 0;
        Form motion_form = new Form();
        bool color = false;
        bool snapshot_color = false;


        public LivePage()
        {
            InitializeComponent();

            //Will start process in MainWindow_Loaded once it is loaded
            this.Loaded += MainWindow_Loaded;
            //Will start process in MainWindow_Closing once it is closed
            this.Closing += MainWindow_Closing;
        }

        void MainWindow_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            _controller.StopProcessing();
        }

        void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            //This is where you have to type in the cameras IP address. The only way I was able to obtain the IP address
            //of the camera was to install the program from the disc that comes with the camera. On that you also have to 
            //set your own username and password. Here capcamera is my username so that is where the username goes and
            //capcamera1 is the password so that is where the password goes.  
            _controller = new IpCamController("http://192.168.1.2", "capcamera", "capcamera1");
            _controller.ImageReady += dec_FrameReady;
            //This function can be found in IpCamController.cs
            _controller.StartProcessing();
            //Gives you the timestamp on the bottom of the page
            time_lable.Content = DateTime.Now.ToString("MM-dd-yyyy h:mmtt");

        }

        void dec_FrameReady(object sender, ImageReadyEventArsgs args)
        {
            Live_Stream.Source = args.Image;
        }


        //Timer that update Base_Pic that that will be compared to the Current Frame. This is where you would go and
        //change the time if you want to make the motion detection more sensitive or less sensitive. 
        void Timer_Ticker(object sender, EventArgs e)
        {

            if (Timer < 3)
            {
                Timer++;
            }

            if (Timer == 3 && color == true)
            {
                //Will make the status box yellow when motion is detected
                motion_box.Fill = new SolidColorBrush(System.Windows.Media.Color.FromArgb(255, 0, 255, 0));
                Timer = 0;
                color = false;
                New_Base_Pic_Timer.Stop();
            }


        }

        //This will set the status box back to the color it was before it turned blue after the user
        //pressed the snapshot button
        void snap_color_timer(object sender, EventArgs e)
        {
            if (Timer_color < 1)
            {
                Timer_color++;
            }

            if (Timer_color == 1 && snapshot_color == true)
            {
                snapshot_box.Fill = new SolidColorBrush(System.Windows.Media.Color.FromArgb(255, 255, 255, 255));
                snapshot_color = false;
                Timer_color = 0;
                snap_color.Stop();
            }

        }

        //The GetHash function will converts images into array of 1's and 0's in order to compare the
        //differences between the two images. In other it will return an array of 1's and 0's that I then use
        //to compare compare two images with each other in order to see if there is motion. 
        public static List<bool> GetHash(Bitmap bmpSource)
        {
            List<bool> lResult = new List<bool>();
            //create new image with 16x16 pixel
            Bitmap bmpMin = new Bitmap(bmpSource, new System.Drawing.Size(16, 16));
            for (int j = 0; j < bmpMin.Height; j++)
            {
                for (int i = 0; i < bmpMin.Width; i++)
                {
                    //reduce colors to true / false                
                    lResult.Add(bmpMin.GetPixel(i, j).GetBrightness() < 0.5f);
                }
            }
            return lResult;
        }


        //Motion Detection starts by turning both Base_Pic and CurrentFrame into Bitmap images so that they can
        //be processed in the GetHash function. Then it tries do deside if the image should be saved or not. 
        void Motion_Detection(object sender, EventArgs e)
        {
            Current_Motion = _controller.CurrentFrame;

            using (MemoryStream outStream1 = new MemoryStream())
            {
                //In order to save and compare images we need to turn them into bitmap form instead of bitmapimage form
                //as the current live feed will give us. This is what we do here.
                BitmapEncoder enc1 = new BmpBitmapEncoder();
                enc1.Frames.Add(BitmapFrame.Create(Current_Motion));
                enc1.Save(outStream1);
                System.Drawing.Bitmap Current_1 = new System.Drawing.Bitmap(outStream1);

                Current_Try = Current_1;
            }

            List<bool> iHash1 = GetHash(new Bitmap(Base_Try));
            
            List<bool> iHash2 = GetHash(new Bitmap(Current_Try));

            //Here we know have an array of 1's and 0's for both images so we can use equalElements in order
            //to compare them. This will return the amount of pixels that are the same. 
            int equalElements = iHash1.Zip(iHash2, (i, j) => i == j).Count(eq => eq);

            DateTime cur = DateTime.Now;
            string Motion = @"C:\TEST\" + cur.Year + cur.Month + cur.Day + cur.Hour + cur.Minute + cur.Second + ".bmp";

            //Create a new bitmap.
            var bmpScreenshot1 = new Bitmap(Screen.PrimaryScreen.Bounds.Width,
                               Screen.PrimaryScreen.Bounds.Height,
                               System.Drawing.Imaging.PixelFormat.Format32bppArgb);
            // Create a graphics object from the bitmap.
            var gfxScreenshot1 = Graphics.FromImage(bmpScreenshot1);

            // Take the screenshot from the upper left corner to the right bottom corner.
            gfxScreenshot1.CopyFromScreen(Screen.PrimaryScreen.Bounds.X,
                                        Screen.PrimaryScreen.Bounds.Y,
                                        0,
                                        0,
                                        Screen.PrimaryScreen.Bounds.Size,
                                        CopyPixelOperation.SourceCopy);

            //This if statment basically saves a screanshot if the amount of pixles in the two images that are the same 
            //is less then 190. If you want it more or less sensitive you have to user a smaller or larger number than
            //190. The number of pixels in one picture is 256 so if two images that look identical are compared 256 will
            //be returned. 
            if (equalElements < 190)
            {
                // Save the screenshot to the specified path that the user has chosen.
                bmpScreenshot1.Save(@"C:\TEST\" + cur.Year + "-" + cur.Month + "-" + cur.Day + " " + cur.Hour + "." + cur.Minute + "." + cur.Second + "Motion" + ".bmp", ImageFormat.Bmp);
                motion_box.Fill = new SolidColorBrush(System.Windows.Media.Color.FromArgb(255, 255, 255, 0));
                color = true;

                //Timer to set base pic
                New_Base_Pic_Timer = new DispatcherTimer();
                New_Base_Pic_Timer.Interval = new TimeSpan(0, 0, 1);
                New_Base_Pic_Timer.Tick += Timer_Ticker;
                New_Base_Pic_Timer.Start();

            }
        }

        
        //Here we save a screenshot when the user click the snapshot button.
        private void Snap_btn_Click(object sender, RoutedEventArgs e)
        {

            DateTime cur = DateTime.Now;
            SnapShot = _controller.CurrentFrame;

            string fileName = @"C:\TEST\" + cur.Year+cur.Month+cur.Day+cur.Hour+cur.Minute+cur.Second +".bmp";

            //Create a new bitmap.
            var bmpScreenshot = new Bitmap(Screen.PrimaryScreen.Bounds.Width,
                               Screen.PrimaryScreen.Bounds.Height,
                               System.Drawing.Imaging.PixelFormat.Format32bppArgb);
            // Create a graphics object from the bitmap.
            var gfxScreenshot = Graphics.FromImage(bmpScreenshot);

            // Take the screenshot from the upper left corner to the right bottom corner.
            gfxScreenshot.CopyFromScreen(Screen.PrimaryScreen.Bounds.X,
                                        Screen.PrimaryScreen.Bounds.Y,
                                        0,
                                        0,
                                        Screen.PrimaryScreen.Bounds.Size,
                                        CopyPixelOperation.SourceCopy);

            using (MemoryStream outStream = new MemoryStream())
            {
                //In order to save and compare images we need to turn them into bitmap form instead of bitmapimage form
                //as the current live feed will give us. This is what we do here.
                BitmapEncoder enc = new BmpBitmapEncoder();
                enc.Frames.Add(BitmapFrame.Create(SnapShot));
                enc.Save(outStream);
                System.Drawing.Bitmap bitmap = new System.Drawing.Bitmap(outStream);

                // Save the screenshot to the specified path that the user has chosen.
                bmpScreenshot.Save(@"C:\TEST\" + cur.Year + "-" + cur.Month + "-" + cur.Day + " " + cur.Hour + "." + cur.Minute + "." + cur.Second + " " + "SnapShot" + ".bmp", ImageFormat.Bmp);
            }

            //We also set the current status box to blue that will than go back to the current color since we
            //established that earlier.
            snapshot_box.Fill = new SolidColorBrush(System.Windows.Media.Color.FromArgb(255, 0, 0, 255));
            snapshot_color = true;

            snap_color = new DispatcherTimer();
            snap_color.Interval = new TimeSpan(0, 0, 1);
            snap_color.Tick += snap_color_timer;
            snap_color.Start();
        }

        //If the user hit the start motion button it will store the current frame image in the base_pic variable.
        //It also starts the motion detection timer.
        private void Set_Base_Pic(object sender, RoutedEventArgs e)
        {

            using (MemoryStream outStream = new MemoryStream())
            {
                //In order to save and compare images we need to turn them into bitmap form instead of bitmapimage form
                //as the current live feed will give us. This is what we do here.
                BitmapEncoder enc = new BmpBitmapEncoder();
                enc.Frames.Add(BitmapFrame.Create(_controller.CurrentFrame));
                enc.Save(outStream);
                System.Drawing.Bitmap Base_1 = new System.Drawing.Bitmap(outStream);

                Base_Try = Base_1;

                //timer to detect motion
                Comp_Images = new DispatcherTimer();
                Comp_Images.Interval = new TimeSpan(0, 0, 5);
                Comp_Images.Tick += Motion_Detection;
                Comp_Images.Start();

                motion_box.Fill = new SolidColorBrush(System.Windows.Media.Color.FromArgb(255, 0, 255, 0));
            }

        }

        //Stop motion is simple bicause it only stops the motion timer and turn the current status box into red.
        private void Stop_Motion(object sender, RoutedEventArgs e)
        {
            Comp_Images.Stop();
            motion_box.Fill = new SolidColorBrush(System.Windows.Media.Color.FromArgb(255, 255, 0, 0));
        }
    }
}
